;;; Model code for Dual Task Timing task Experiment 2 from
;;;
;;; N.A. Taatgen, H. van Rijn, J.R. Anderson. An Integrated theory of prospective time estimation:
;;; 
;;; (do-all n) runs the model n times for each of the four conditions and creates detailed output files


;;; (do-all n) runs the model n times for each of the four conditions and creates detailed output files
;;; in the MCL directory
;;; These files can be analyzed using a whole bunch of functions in this file, most of which are obsolete.
;;; The functions I used for the final analysis are the following:

;;; Analysis functions are:
;;; (time-first-only *all* fname)
;;; This will write all the click-times relative to the optimal click-time to the file you specify with fname
;;; If you don't runs this function right after running the model, first type (set-vars n), with n the number
;;; of virtual subjects (this sets the *all* variable appropriately)
;;;
;;; The dual-tasking scores can be obtained by
;;; (dual-indiv *all*)
;;; 
;;; The scores for each block can be obtained by
;;; (scores-indiv *all*)
;;; This latter call produces a list that has to be pasted as a column to the right of
;;; the output of the dual-indiv call.
;;;
;;; You can also use the code to run the experiment with a subject
;;; use (do-ll x), (do-la x), (do-al x) or (do-aa x) with x the subject number

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; This is the code that runs the experiment
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar *money* 0.0)
(defvar money-object nil)
(defvar happy-object nil)
(defvar earn-button)
(defvar spend-button)
(defvar earn-hit-present)
(defvar earn-button-status)
(defvar earn-button-present)
(defvar show-buttons)
(defvar *start-pressed*)
(defvar *window*)

(defvar *response*)
(defvar *update-money*)
(defvar *log* nil)
(defvar *tt*)
(defvar *vowels* '("A"))
(defvar *consonants* '("B"))

(defvar *add-correct*)
(defvar *add-error*)
(setf *add-correct* nil *add-error* nil)
(defvar *hit-list*)
(defvar *foil-list*)
(defvar **visual-error**)


(dotimes (i 8)
  (dotimes (j (- 8 i))
    (push (format nil "~D+~D=~D" (1+ i) (1+ j) (+ i j 2)) *add-correct*)
    (if (< (+ i j 3) 10)
      (push (format nil "~D+~D=~D" (1+ i) (1+ j) (+ i j 3)) *add-error*)
      (push (format nil "~D+~D=~D" (1+ i) (1+ j) (+ i j 1)) *add-error*)
      )))
(defun random-member (l)
  (nth (random (length l)) l))

(defun report (event)
  (if *actr-enabled-p*
      (push (list (* (- (actr-time) *tt*) 1000) event) *log*)
      (push (list (- (get-internal-real-time) *tt*) event) *log*)))

(defun add-box-to-exp-window (&key (x 0)(y 0)(height 50)(width 50))
  (add-line-to-exp-window (list x y)(list (+ x width) y))
  (add-line-to-exp-window (list (+ x width) y) (list (+ x width) (+ y height)))
  (add-line-to-exp-window (list (+ x width) (+ y height)) (list x (+ y height)))
  (add-line-to-exp-window (list x (+ y height)) (list x y))) 
                                   
(defun update-money ()
  (when (null *actr-enabled-p*)
  (let ((xo (round (/ (- *screen-width* 800) 2)))
        (yo (round (/ (- *screen-height* 350) 2))))
    (when money-object (remove-items-from-exp-window money-object))
    (setf money-object (add-text-to-exp-window 
                        :text (format nil "  ~6F" *money*)
                        :x (+ xo 390) :y (+ yo 165))))
))


(defun score-button (button)
  (setf earn-button nil earn-hit-present nil)
  (remove-items-from-exp-window button)
  (if show-buttons (incf *money* 100) (incf *money* 30))
  (report 'key-earn-score)
  (when (not *actr-enabled-p*) (beep))
  (if *actr-enabled-p* (pm-proc-display) (update-money))
)

(defun foil-button (button)
  (setf earn-button nil)  
  (remove-items-from-exp-window button)
  (setf *money* (max 0 (- *money* (if show-buttons 100 30))))
  (report 'key-earn-error)
  (if *actr-enabled-p* (pm-proc-display) (update-money))
)


(defun spend-button (button)
  (setf spend-button nil)
  (incf *money* 30)
  (report 'click-spend-score)
  (remove-items-from-exp-window button)
  (if *actr-enabled-p* (pm-proc-display) (setf *update-money* t))
)

(defun foilspend-button (button)
   (setf spend-button nil)
  (setf *money* (max 0 (- *money* 30)))
  (report 'click-spend-error)
  (remove-items-from-exp-window button)
  (if *actr-enabled-p* (pm-proc-display) (setf *update-money* t))
)

(defun gen-slow-random ()
  (if (= (random 2) 1)
    (random 200)
    (+ 500 (random 200))))

(defun generate-event-list (total-duration earn-events duration lag earn-pause)
  (let ((result nil))
    (do* (
          (slow-random-count 7 (if (= slow-random-count 0) (+ 5 (random 3))(1- slow-random-count)))
          (slow-random (gen-slow-random) (if (= slow-random-count 1) (gen-slow-random) slow-random))
          (tt 0 (+ tt duration 100 slow-random (random 300)))
          (start-event nil (list tt (if (> (random 1.0) 0.4) 'spend-hit 'spend-foil)))
          (end-event nil (list (+ tt duration) (if (eq (second start-event) 'spend-hit) 'spend-hit-done 'spend-foil-done))))
         ((> tt total-duration))
      (when start-event (push start-event result)
            (push end-event result)))
    (do* (
          (slow-random-count 7 (if (= slow-random-count 0) (+ 5 (random 3))(1- slow-random-count)))
          (slow-random (gen-slow-random) (if (= slow-random-count 1) (gen-slow-random) slow-random))
          (tt 0 (+ tt duration 100 slow-random (random 300)))
          (start-event nil (list tt (if (> (random 1.0) 0.4) 'earn-hit 'earn-foil)))
          (end-event nil (list (+ tt duration) (if (eq (second start-event) 'earn-hit) 'earn-hit-done 'earn-foil-done))))
         ((> tt total-duration))
      (when start-event (push start-event result)
            (push end-event result)))
    (do* (
          (tt 200 (+ tt 13000)))
         ((> tt total-duration))
      (push (list tt 'start-earn) result)
      (push (list (+ tt 6000) 'stop-earn) result)
      (push (list (+ tt 6500) 'remove-notice) result))
    (push (list total-duration 'stop) result)
    (sort result #'(lambda (x y) (< (first x)(first y))))))
         
(defun random-earn-button (txt action)
  (let* ((xo (if *actr-enabled-p* 0 (round (/ (- *screen-width* 800) 2))))
         (yo (if *actr-enabled-p* 0 (round (/ (- *screen-height* 350) 2))))
         (x (+ xo 22 (random 126)))
         (y (+ yo 52 (random 196)))
         button)
    (report (list 'display-earn-button (- x 22 xo) (- y 52 yo) txt))
    (setf button (add-button-to-exp-window :text txt :x x :y y :height 50 :width 120 :action action))
    (when *actr-enabled-p* (pm-proc-display))
    button))
      
(defun random-spend-button (txt action)
  (let* ((xo (if *actr-enabled-p* 0 (round (/ (- *screen-width* 800) 2))))
        (yo (if *actr-enabled-p* 0 (round (/ (- *screen-height* 350) 2))))
        (x (+ 532 xo (random 126)))
        (y (+ 52 yo (random 196)))
        button)
    (report (list 'display-spend-button (- x 532 xo) (- y 52 yo) txt))
    (setf button (add-button-to-exp-window :text txt :x x :y y :height 50 :width 120 :action action))
    (when *actr-enabled-p* (pm-proc-display))
    button))

(defun press-test (button)
  (report 'press-test)
  (setf *money* (max 0 (- *money* 10)))
  (when (null *actr-enabled-p*) (update-money)
        (setf *update-money* t))
  (when (and earn-button-status (null show-buttons))
    (setf show-buttons t)
    (let ((xoffset (if *actr-enabled-p* 0 (round (/ (- *screen-width* 800) 2))))
          (yoffset (if *actr-enabled-p* 0 (round (/ (- *screen-height* 350) 2)))))
      (setf high-text (add-text-to-exp-window :text "HIGH" :width 180 :x (+ xoffset 100) :y (+ yoffset 20))))
    ;(when (eq earn-button-present 'hit) (setf earn-button (random-earn-button (random-member *hit-list*) nil))
    ;      (setf earn-hit-present t))
    ;(when (eq earn-button-present 'foil) (setf earn-button (random-earn-button (random-member *foil-list*) nil)))
    )
)

(defmethod rpm-window-key-event-handler ((win rpm-window) key)
  (if *actr-enabled-p*
      (if (and earn-button earn-hit-present) (score-button earn-button)
          (if earn-button (foil-button earn-button)
              (report 'key-earn-non-present)))
      (setf *response* t))
)

(defun start-pressed (button)
  (setf *tt* (get-internal-real-time))
  (setf *start-pressed* t)
  (remove-items-from-exp-window button)
)

(defvar *addition*)

(defun set-letters ()
  (setf *hit-list* *vowels*)
  (setf *foil-list* *consonants*)
  (setf *addition* nil))

(defun set-addition-model ()
  (setf *hit-list* '("E"))
  (setf *foil-list* '("D"))
  (setf *addition* t))

(defun set-addition ()
  (setf *hit-list* *add-correct*)
  (setf *foil-list* *add-error*))
 

(defun do-trial (&optional (x 1))
  (let* ((xoffset (round (/ (- *screen-width* 800) 2)))
         (yoffset (round (/ (- *screen-height* 350) 2)))
         (window (open-exp-window "Points" :visible t :x 0 :y 0 :width *screen-width* :height *screen-height*))
         (Money (add-text-to-exp-window :text "Score" :x (+ xoffset 380) :y (+ yoffset 145)))
         (start1 (add-text-to-exp-window :text (format nil "This is block ~D (out of 15)" x)
                                         :x (+ xoffset 40) :width 400
                                         :y (+ yoffset 340))) 
         (start2 (add-text-to-exp-window :text 
                                         (if (equal *hit-list* *vowels*)
                                           "Your task is to respond to the letter A"
                                           "Your task is to respond to correct additions")
                                         :x (+ xoffset 40) :width 400
                                         :y (+ yoffset 360)))    
         (earn (add-text-to-exp-window :text "High profit" :x (+ xoffset 200) :y (+ yoffset 20)))
         (spend (add-text-to-exp-window :text "Low profit" :x (+ xoffset 700) :y (+ yoffset 20)))
         (earn-box (add-box-to-exp-window  :x (+ xoffset 20) :y (+ yoffset 50) :height 250 :width 250))
         (spend-box (add-box-to-exp-window :x (+ xoffset 530) :y (+ yoffset 50) :height 250 :width 250))
         (test-button (add-button-to-exp-window :text "Test High" :x (+ xoffset 335) :y (+ yoffset 50) :width 130 :height 30 :action #'press-test))
         (start-button (add-button-to-exp-window :text "Start" :x (+ xoffset 350) :y (+ yoffset 100) :width 100 :height 30 :action #'start-pressed))
         (event-list (generate-event-list 120000 3 1200 300 7000))
         (now-time 0)
         notice
    )
    (pm-install-device window)
  (setf show-buttons nil money-object nil happy-object nil earn-button nil 
        spend-button nil *money* 0 earn-button-status nil *window* window)
  (update-money)
  (setf *start-pressed* nil)
  (setf *log* nil)
  (setf *response* nil)
  (setf *update-money* nil)
  (while (null *start-pressed*) (allow-event-manager window))
  (pm-proc-display)
  (dolist (x event-list)
    (when (null *actr-enabled-p*) (setf now-time (- (get-internal-real-time) *tt*)))
;    (format t "~%~S ~6D ~6D" (second x) (first x) now-time)
    (while (> (- (first x) now-time) 0) 
      (allow-event-manager *window*)
      (when *response* 
        (if (and earn-button earn-hit-present) (score-button earn-button)
            (if earn-button (foil-button earn-button)
                (report 'key-earn-non-present)))
        (setf *response* nil))
      (when *update-money* (update-money)(setf *update-money* nil))
      (setf now-time (- (get-internal-real-time) *tt*))
      )
    (cond
     ((eq (second x) 'earn-hit) 
      	(setf earn-button (random-earn-button (random-member *hit-list*) nil)
        	                     earn-hit-present t)
        	  (setf earn-button-present 'hit))
     ((eq (second x) 'earn-foil) 
        (setf earn-button (random-earn-button (random-member *foil-list*) nil))
            (setf earn-button-present 'foil)
        )
     ((eq (second x) 'earn-hit-done)
      (cond (earn-button
             (setf earn-hit-present nil)
             (remove-items-from-exp-window earn-button)
             (report 'remove-earn-hit-button))
            )        
      (setf earn-button-present nil)
      (setf earn-button nil)
      )
     ((eq (second x) 'earn-foil-done)
      (setf earn-button-present nil)
      (cond (earn-button (remove-items-from-exp-window earn-button)
             (report 'remove-earn-foil-button))
            ((null show-buttons) (report 'missed-earn-foil)))
      (setf earn-hit-present nil)
      (setf earn-button nil)
      )
     ((eq (second x) 'spend-hit) (setf spend-button (random-spend-button (random-member *hit-list*) #'spend-button)))
     ((eq (second x) 'spend-foil) (setf spend-button (random-spend-button (random-member *foil-list*) #'foilspend-button)))
     ((and (eq (second x) 'spend-hit-done) spend-button)
      (report 'remove-spend-hit-button)
      (remove-items-from-exp-window spend-button)
      (setf spend-button nil))
     ((and (eq (second x) 'spend-foil-done) spend-button)
      (report 'remove-spend-foil-button)
      (remove-items-from-exp-window spend-button)
      (setf spend-button nil))   
     ((eq (second x) 'start-earn) (setf earn-button-status t)(report 'start-earn-series))
     ((eq (second x) 'stop-earn) 
      (report 'end-earn-series)
      (setf earn-button-status nil show-buttons nil)
      (when  high-text (remove-items-from-exp-window high-text)(setf high-text nil))
      (setf notice (add-text-to-exp-window :text "End High profit" :x (+ xoffset 100) :y (+ yoffset 150) :width 180)))
     ((eq (second x) 'remove-notice) (remove-items-from-exp-window notice)))
     (allow-event-manager window))
  (clear-exp-window)
  (setf money-object nil)
  (add-text-to-exp-window :text "Final Score" :x (+ xoffset 380) :y (+ yoffset 150) :width 250)
  (update-money)
  (sleep 5.0)
  (close-exp-window)
  (report (list 'final-scores *money*))
  ))

(defun do-trial-model (&optional (x 1))
  (setf old-time (mp-time *mp*))
  (pm-reset)
  (set-clock *mp* old-time)
  (setf *time* old-time)
  (gc)
  (let* ((xoffset 0)
         (yoffset 0)
         (window (open-exp-window "Points" :visible nil :x 0 :y 0 :width 800 :height 350))
         (test-button (add-button-to-exp-window :text "Test" :x (+ xoffset 335) :y (+ yoffset 50) :width 130 :height 30 :action #'press-test))
         (event-list (generate-event-list 120000 3 1200 300 7000))
         notice
    )
    (setf *actr-enabled-p* t)
    (pm-set-params :auto-dequeue t
                   :real-time nil
                   :show-focus t
                   :VISUAL-FINST-SPAN 10000.0
                   :VISUAL-NUM-FINSTS 1000
                   :mouse-fitts-coeff .0483) ; 0.0483 this is instead of the default of .1 to compensate virtual windows (?)
    (pm-start-hand-at-mouse)
    (pm-install-device window)
    (pm-set-visloc-default :attended nil :kind text  :screen-x lowest)
  (setf show-buttons nil money-object nil happy-object nil earn-button nil 
        spend-button nil *money* 0 earn-button-status nil *window* window)
  (setf (last-cmd (motor-m *mp*)) nil)
  (setf **visual-error** nil)
  (setf *visual-location* nil) 
  (setf *start-pressed* nil)
  (setf *log* nil)
  (setf *response* nil)
  (setf *update-money* nil)
  (setf *last-score* 0)
  (pm-proc-display)
  (setf *tt* (actr-time))
  (dolist (x event-list)
    (pm-run (- (/ (first x) 1000) (- (actr-time) *tt*)) :full-time t)
    (cond
     ((eq (second x) 'earn-hit) 
      	(setf earn-button (random-earn-button (random-member *hit-list*) nil)
        	                     earn-hit-present t)
        	  (setf earn-button-present 'hit))
     ((eq (second x) 'earn-foil) 
        (setf earn-button (random-earn-button (random-member *foil-list*) nil))
            (setf earn-button-present 'foil)
        )
     ((eq (second x) 'earn-hit-done)
      (cond (earn-button
             (setf earn-hit-present nil)
             (remove-items-from-exp-window earn-button)
             (report 'remove-earn-hit-button))
            )     
      (setf earn-button-present nil)
      (setf earn-button nil)
      )
     ((eq (second x) 'earn-foil-done)
      (setf earn-button-present nil)
      (cond (earn-button (remove-items-from-exp-window earn-button)
             (report 'remove-earn-foil-button))
            ((null show-buttons) (report 'missed-earn-foil)))
      (setf earn-hit-present nil)
      (setf earn-button nil)
      )
     ((eq (second x) 'spend-hit) (setf spend-button (random-spend-button (random-member *hit-list*) #'spend-button)))
     ((eq (second x) 'spend-foil) (setf spend-button (random-spend-button (random-member *foil-list*) #'foilspend-button)))
     ((and (eq (second x) 'spend-hit-done) spend-button)
      (report 'remove-spend-hit-button)
      (remove-items-from-exp-window spend-button)
      (setf spend-button nil))
     ((and (eq (second x) 'spend-foil-done) spend-button)
      (report 'remove-spend-foil-button)
      (remove-items-from-exp-window spend-button)
      (setf spend-button nil))   
     ((eq (second x) 'start-earn) (setf earn-button-status t)(report 'start-earn-series))
     ((eq (second x) 'stop-earn) 
      (report 'end-earn-series)
      (setf earn-button-status nil show-buttons nil)
      (when  high-text (remove-items-from-exp-window high-text)(setf high-text nil))
      (setf notice (add-text-to-exp-window :text "End" :x (+ xoffset 100) :y (+ yoffset 150) :width 180))
      (pm-proc-display))
     ((eq (second x) 'remove-notice) (remove-items-from-exp-window notice)))
     (pm-proc-display))
  (clear-exp-window)
;  (setf money-object nil)
;  (add-text-to-exp-window :text "Final Score" :x (+ xoffset 380) :y (+ yoffset 150) :width 250)
;  (update-money)
;  (sleep 5.0)
  (close-exp-window)
  (report (list 'final-scores *money*))
  ))

(defun show-report (&optional (log *log*))
  (dolist (i (reverse log))
    (format t "~8D ~S~%" (first i) (second i))))



(defun do-la (n &optional (m 1))
   (setf *actr-enabled-p* nil)
 (set-letters)
  (dotimes (i 15)
    (unless (< i (- m 1))
      (when (or (= i 5)(= i 10))
        (message-dialog (format nil "~% You can take a brief break now. ~%~% Click OK when you are ready to continue") :size #@(670 200)))
      (when (= i 10) (set-addition)
            (message-dialog (format nil "~% Note that the task is changed for the last five trials: ~%~% You now have to respond to correct additions.~%~% Click OK to continue")
                            :size #@(670 200))
            )
      (do-trial (1+ i))
      (with-open-file (f (format nil "MoneyExp1-S~D-T~D-la" n (1+ i)) :direction :output :if-exists :error)
        (format f "(setf *log* '~S)" *log*)))))

(defun do-al (n &optional (m 1))
   (setf *actr-enabled-p* nil)
 (set-addition)
  (dotimes (i 15)
    (unless (< i (- m 1))
      (when (or (= i 5)(= i 10))
        (message-dialog (format nil "~% You can take a brief break now. ~%~% Click OK when you are ready to continue") :size #@(670 200)))
      (when (= i 10) (set-letters)
            (message-dialog (format nil "~% Note that the task is changed for the last five trials: ~%~% You now have to respond to the letter A.~%~% Click OK to continue")
                            :size #@(670 200))
            )    
      (do-trial (1+ i))
      (with-open-file (f (format nil "MoneyExp1-S~D-T~D-al" n (1+ i)) :direction :output :if-exists :error)
        (format f "(setf *log* '~S)" *log*)))))

(defun do-ll (n &optional (m 1))
  (setf *actr-enabled-p* nil)
  (set-letters)
  (dotimes (i 15)
    (unless (< i (- m 1))
      (when (or (= i 5)(= i 10))
        (message-dialog (format nil "~% You can take a brief break now. ~%~% Click OK when you are ready to continue") :size #@(670 200)))
      (do-trial (1+ i))
      (with-open-file (f (format nil "MoneyExp1-S~D-T~D-ll" n (1+ i)) :direction :output :if-exists :error)
        (format f "(setf *log* '~S)" *log*)))))

(defun do-aa (n &optional (m 1))
  (setf *actr-enabled-p* nil)
  (set-addition)
  (dotimes (i 15)
    (unless (< i (- m 1))
      (when (or (= i 5)(= i 10))
        (message-dialog (format nil "~% You can take a brief break now. ~%~% Click OK when you are ready to continue") :size #@(670 200)))
      (do-trial (1+ i))
      (with-open-file (f (format nil "MoneyExp1-S~D-T~D-aa" n (1+ i)) :direction :output :if-exists :error)
        (format f "(setf *log* '~S)" *log*)))))

(defun do-ll-model (n &key (v nil))
  (reload)
  (set-letters)
  (pm-reset)
  (start-subject)
  (eval `(sgp :v ,v))
  (dotimes (i 20)
    (let ((goal (new-name "GOAL")))
      (eval `(add-dm (,goal isa goal state start )))
      (eval `(goal-focus ,goal)))
    (do-trial-model (1+ i))
    (with-open-file (f (format nil "MoneyExp1-S~D-T~D-ll" (+ 100 n) (1+ i)) :direction :output :if-exists :rename-and-delete)
      (format f "(setf *log* '~S)" *log*))))

(defun do-aa-model (n &key (v nil))
  (reload)
  (set-addition-model)
  (pm-reset)
  (start-subject)
  (eval `(sgp :v ,v))
  (spp (decide-response-left decide-no-response-left decide-no-response-left-2 
                            decide-response-right decide-no-response-right decide-no-response-right-2)
       :effort .3) ;;; estimate to retrieve an addition: 300 ms
  (dotimes (i 20)
    (let ((goal (new-name "GOAL")))
      (eval `(add-dm (,goal isa goal state start )))
      (eval `(goal-focus ,goal)))
    (do-trial-model (1+ i))
    (with-open-file (f (format nil "MoneyExp1-S~D-T~D-aa" (+ 100 n) (1+ i)) :direction :output :if-exists :rename-and-delete)
      (format f "(setf *log* '~S)" *log*))))

(defun do-al-model (n &key (v nil))
  (reload)
  (set-addition-model)
  (pm-reset)
  (start-subject)
  (eval `(sgp :v ,v))
  (spp (decide-response-left decide-no-response-left decide-no-response-left-2 
                            decide-response-right decide-no-response-right decide-no-response-right-2)
       :effort .3) ;;; estimate to retrieve an addition: 300 ms
  (dotimes (i 20)
    (when (= i 10)
      (set-letters)
   (spp (decide-response-left decide-no-response-left decide-no-response-left-2 
                            decide-response-right decide-no-response-right decide-no-response-right-2)
       :effort .05)
      ) ;;; default
   (let ((goal (new-name "GOAL")))
      (eval `(add-dm (,goal isa goal state start )))
      (eval `(goal-focus ,goal)))
    (do-trial-model (1+ i))
    (with-open-file (f (format nil "MoneyExp1-S~D-T~D-al" (+ 100 n) (1+ i)) :direction :output :if-exists :rename-and-delete)
      (format f "(setf *log* '~S)" *log*))))

(defun do-la-model (n &key (v nil))
  (reload)
  (set-letters)
  (pm-reset)
  (start-subject)
  (eval `(sgp :v ,v))
  (dotimes (i 20)
    (when (= i 10)
      (set-addition-model)
   (spp (decide-response-left decide-no-response-left decide-no-response-left-2 
                            decide-response-right decide-no-response-right decide-no-response-right-2)
       :effort .3)
      ) ;;; default
   (let ((goal (new-name "GOAL")))
      (eval `(add-dm (,goal isa goal state start)))
      (eval `(goal-focus ,goal)))
    (do-trial-model (1+ i))
    (with-open-file (f (format nil "MoneyExp1-S~D-T~D-la" (+ 100 n) (1+ i)) :direction :output :if-exists :rename-and-delete)
      (format f "(setf *log* '~S)" *log*))))

(defun do-all (n &optional (m 1))
  (dotimes (i n)
    (format t "~%~%**** Run ~S ****~%~%" (+ i m))
    (do-ll-model (+ m i))
    (do-aa-model (+ m i))
    (do-la-model (+ m i))
    (do-al-model (+ m i)))
  (set-vars (+ m n -1))
  (display-all))

(defun display-all ()
  (histo+ *ll*)
  (scores *ll*)
  (dual *ll*)
  (histo+ *aa*)
  (scores *aa*)
  (dual *aa*)
  (histo+ *la*)
  (scores *la*)
  (dual *la*)
  (histo+ *al*)
  (scores *al*)
  (dual *al*))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Code to analyse results
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun find-test-begin-differences (&optional (log *log*))
  (let ((start-earn -100000) (test-list nil))
    (dolist (i (reverse log))
      (when (eq (second i) 'press-test) (push (first i) test-list))
      (when (eq (second i) 'start-earn-series)
        (dolist (x (reverse test-list))
          (format t "Previous earn ~D  Test-moment ~D Current earn ~D~%" start-earn x (first i))
           (format t " ~6D ~6D~%" x
                  (if (< (- x start-earn) (- (first i) x))
                    (- x start-earn)
                    (- x (first i)))
                 ))
        (setf test-list nil)
        (setf start-earn (first i))))))

(defun find-stats (&optional (log *log*))
  (let ((total-earn-hit 0)
        (total-spend-hit 0)
        (total-earn-foil 0)
        (total-spend-foil 0)
        (earn-hit-correct 0)
        (earn-foiled 0)
        (spend-hit-correct 0)
        (spend-foiled 0)
        (total-spend-hit-during-earn 0)
        (total-spend-foil-during-earn 0)
        (spend-hit-correct-during-earn 0)
        (spend-foiled-during-earn 0)
        (earn-period nil)
        )
    (dolist (i (Reverse log))
      (cond
       ((eq (second i) 'key-earn-error) (incf earn-foiled)(incf total-earn-foil))
       ((eq (second i) 'key-earn-score) (incf earn-hit-correct)(incf total-earn-hit))
       ((eq (second i) 'start-earn-series) (setf earn-period 'inited))
       ((eq (second i) 'press-test) (when (eq earn-period 'inited)(setf earn-period 'yes)))
       ((eq (second i) 'end-earn-series) (setf earn-period nil))
       ((eq (second i) 'earn-hit-done)(incf total-earn-hit))
       ((eq (second i) 'earn-foil-done)(incf total-earn-foil))
       ((eq (second i) 'remove-earn-foil-button)(incf total-earn-foil))
       ((eq (second i) 'remove-earn-hit-button)(incf total-earn-hit))
       ((eq (second i) 'click-spend-error) (incf spend-foiled)(incf total-spend-foil)
        (when (eq earn-period 'yes)(incf total-spend-foil-during-earn)(incf spend-foiled-during-earn)))
       ((eq (second i) 'click-spend-score)(incf spend-hit-correct)(incf total-spend-hit)
        (when (eq earn-period 'yes) (incf total-spend-hit-during-earn)(incf spend-hit-correct-during-earn)))
       ((eq (second i) 'remove-spend-foil-button)(incf total-spend-foil)
        (when (eq earn-period 'yes) (incf total-spend-foil-during-earn))) 
       ((eq (second i) 'remove-spend-hit-button)(incf total-spend-hit)
        (when (eq earn-period 'yes) (incf total-spend-hit-during-earn)))))
;    (format t "Earning:~%")
    (format t "~4D ~4D ~6,3F " earn-hit-correct total-earn-hit (if (> total-earn-hit 0) (/ earn-hit-correct total-earn-hit) 0))
    (format t "~4D ~4D ~6,3F " earn-foiled total-earn-foil (if (> total-earn-foil 0)(/ earn-foiled total-earn-foil) 0))
;    (format t "~%Spending:~%")
    (format t "~4D ~4D ~6,3F " spend-hit-correct total-spend-hit (if (> total-spend-hit 0)(/ spend-hit-correct total-spend-hit) 0))
    (format t "~4D ~4D ~6,3F " spend-foiled total-spend-foil (if (> total-spend-foil 0)(/ spend-foiled total-spend-foil) 0))
;    (format t "~4D ~4D ~6,3F ")
    (format t "~4D ~4D ~6,3F " spend-hit-correct-during-earn total-spend-hit-during-earn 
            (if (> total-spend-hit-during-earn 0) (/ spend-hit-correct-during-earn total-spend-hit-during-earn) 0))
    (format t "~4D ~4D ~6,3F " spend-foiled-during-earn total-spend-foil-during-earn 
            (if (> total-spend-foil-during-earn 0) (/ spend-foiled-during-earn total-spend-foil-during-earn) 0))
    (format t "~%")
))
       
(defun analyze-histo (n c b?)
  (dotimes (i 3)
    (let ((bins (make-array '(25) :initial-element 0)))
      (dotimes (j 5)
        (load (format nil "MoneyExp1-S~D-T~D-~S" n (+ (* i 5) j 1) c))
        (let ((start-earn -100000) (test-list nil) value)
          (dolist (i (reverse *log*))
            (when (eq (second i) 'press-test) (push (first i) test-list))
            (when (eq (second i) 'start-earn-series)
              (dolist (x (reverse test-list))
                (setf value
                      (if (< (- x start-earn) (- (first i) x))
                        (- x start-earn)
                        (- x (first i)))
                      )
                (let ((bin (truncate (/ (+ value 6250) 500))))       
                  (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                            ))
                  ))
              (setf test-list nil)
              (setf start-earn (first i))))))
      (format t "~%Block ~D~%" (1+ i))
      (dotimes (j 25)
        (if b?
          (format t " ~8D ~5D~%" (- (* j 500) 6000) (aref bins j))
          (format t " ~5D~%" (aref bins j))

)))))

(defun analyze (n c)
  (dotimes (i 15)
    (load (format nil "MoneyExp1-S~D-T~D-~S" n (1+ i) c))
    (find-stats))
  (analyze-histo n c t)
  (dotimes (i 15)
    (load (format nil "MoneyExp1-S~D-T~D-~S" n (1+ i) c))
    (find-test-begin-differences)(format t "~%")))

(defun print-log ()
  (dolist (i (reverse *log*))
    (print i)))

(defun histo-multi (l)
  (let ((total (make-array '(25 3) :initial-element 0)))
    (dolist (x l)
      (dotimes (i 3)
        (let ((bins (make-array '(25) :initial-element 0))
              (count 0))
          (dotimes (j 5)
            (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ (* i 5) j 1) (second x)))
            (let ((start-earn -100000) (test-list nil) value)
              (dolist (i (reverse *log*))
                (when (eq (second i) 'press-test) (push (first i) test-list))
                (when (eq (second i) 'start-earn-series)
                  (dolist (x (reverse test-list))
                    (setf value
                          (if (< (- x start-earn) (- (first i) x))
                            (- x start-earn)
                            (- x (first i)))
                          )
                    (let ((bin (truncate (/ (+ value 6250) 500))))       
                      (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                                  ))
                      (incf count)
                  ))
                  (setf test-list nil)
                  (setf start-earn (first i))))))
          (dotimes (j 25)
            (setf (aref total j i) (+ (aref total j i) (/ (aref bins j) count))))
          )))
    (dotimes (i 3)
      (format t "~%Block ~D~%" (1+ i))
      (dotimes (j 25)
        (format t " ~8,4F~%" (/ (aref total j i) (length l)))))))

(defun scores (l)
  (let ((total (make-array '(20) :initial-element 0)))
    (dolist (x l)
      (dotimes (i 20)
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ i 1) (second x)))
        (setf (aref total i) (+ (aref total i) (second (second (first *log*)))))))
    (dotimes (i 20)
      (format t "~8,2F~%" (/ (aref total i) (length l))))))

(defun avg-score (l)
  (let ((total (make-array '(20) :initial-element 0))
        (avg-s 0))
    (dolist (x l)
      (dotimes (i 20)
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ i 1) (second x)))
        (setf (aref total i) (+ (aref total i) (second (second (first *log*)))))))
    (dotimes (i 20) (setf avg-s (+ avg-s (/ (aref total i)(length l)))))
    (format t "~6,1F~%" (/ avg-s 20))))

;;; How often is the right box used during a high profit period
(defun dual (l)
  (let ((total (make-array '(20) :initial-element 0)))
    (dolist (x l)
      (dotimes (j 20)
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ j 1) (second x)))
        (let ((total-earn-hit 0)
              (total-spend-hit 0)
              (total-earn-foil 0)
              (total-spend-foil 0)
              (earn-hit-correct 0)
              (earn-foiled 0)
              (spend-hit-correct 0)
              (spend-foiled 0)
              (total-spend-hit-during-earn 0)
              (total-spend-foil-during-earn 0)
              (spend-hit-correct-during-earn 0)
              (spend-foiled-during-earn 0)
              (earn-period nil)
              )
          (dolist (i (Reverse *log*))
            (cond
             ((eq (second i) 'key-earn-error) (incf earn-foiled)(incf total-earn-foil))
             ((eq (second i) 'key-earn-score) (incf earn-hit-correct)(incf total-earn-hit))
             ((eq (second i) 'start-earn-series) (setf earn-period 'inited))
             ((eq (second i) 'press-test) (when (eq earn-period 'inited)(setf earn-period 'yes)))
             ((eq (second i) 'end-earn-series) (setf earn-period nil))
             ((eq (second i) 'earn-hit-done)(incf total-earn-hit))
             ((eq (second i) 'earn-foil-done)(incf total-earn-foil))
             ((eq (second i) 'remove-earn-foil-button)(incf total-earn-foil))
             ((eq (second i) 'remove-earn-hit-button)(incf total-earn-hit))
             ((eq (second i) 'click-spend-error) (incf spend-foiled)(incf total-spend-foil)
              (when (eq earn-period 'yes)(incf total-spend-foil-during-earn)(incf spend-foiled-during-earn)))
             ((eq (second i) 'click-spend-score)(incf spend-hit-correct)(incf total-spend-hit)
              (when (eq earn-period 'yes) (incf total-spend-hit-during-earn)(incf spend-hit-correct-during-earn)))
             ((eq (second i) 'remove-spend-foil-button)(incf total-spend-foil)
              (when (eq earn-period 'yes) (incf total-spend-foil-during-earn))) 
             ((eq (second i) 'remove-spend-hit-button)(incf total-spend-hit)
              (when (eq earn-period 'yes) (incf total-spend-hit-during-earn)))))
          (setf (aref total j) (+ (aref total j) 
                                  (if (> total-spend-hit-during-earn 0) (/ spend-hit-correct-during-earn total-spend-hit-during-earn) 0))))))
    (dotimes (i 20)
      (format t "~8,2F~%" (/ (aref total i) (length l))))))

;;; How often are two targets that are on the screen at the same time scored both?

(defun test-true (s)
  (cond ((equal s "A") t)
        ((equal s "B") nil)
        (t 
         (let ((add1 (parse-integer (subseq s 0 1)))
               (add2 (parse-integer (subseq s 2 3)))
               (add3 (parse-integer (subseq s 4 5))))
           (= (+ add1 add2) add3)))))
           


(defun average-time (l)
  (let ((total (make-array '(15) :initial-element 0))
        (total-abs (make-array '(15) :initial-element 0))
        (total-sd (make-array '(15) :initial-element 0))
        (total-sd-abs (make-array '(15) :initial-element 0))
        )
    (dolist (x l)
      (dotimes (j 15)
        (let ((accum 0)(accum-abs 0)
              (count 0))
          (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ j 1) (second x)))
          (let ((start-earn -100000) (test-list nil) value)
            (dolist (i (reverse *log*))
              (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
              (when (eq (second i) 'start-earn-series)
                (dolist (x (reverse test-list))
                  (setf value
                        (if (< (- x start-earn) (- (first i) x))
                          (- x start-earn)
                          (- x (first i)))
                        )
                  (incf count)
                  (setf accum (+ accum value)
                        accum-abs (+ accum-abs (if (> value 0) value (- value))))
                  )
                  (setf test-list nil)
                  (setf start-earn (first i)))))
          (setf (aref total j) (+ (aref total j) (/ accum count)))
          (setf (aref total-sd j)(+ (aref total-sd j) (* (/ accum count)(/ accum count))))
          (setf (aref total-abs j)(+ (aref total-abs j)(/ accum-abs count)))
          (setf (aref total-sd-abs j)(+ (aref total-sd-abs j)(* (/ accum-abs count)(/ accum-abs count))))
          )))
    (dotimes (j 15)
      (setf (aref total j) (/ (aref total j) (length l)))
      (setf (aref total-abs j)(/ (aref total-abs j)(length l)))
      (setf (aref total-sd-abs j) (sqrt (- (/ (aref total-sd-abs j) (length l)) (* (aref total-abs j)(aref total-abs j)))))
      (setf (aref total-sd j)(sqrt (- (/ (aref total-sd j) (length l)) (* (aref total j)(aref total j)))))
      (format t " ~8,2F  ~8,2F  ~8,2F  ~8,2F~%" (aref total j)(aref total-sd j)(aref total-abs j)(aref total-sd-abs j)))))

(defun average-time-indiv (l &optional (blocks 20))
  (dolist (x l)
    (format t "~%~5D ~4S " (first x) (second x))
    (dotimes (j 15)
      (let ((accum blocks))
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ j 1) (second x)))
        (let ((start-earn -100000) (test-list nil) value)
          (dolist (i (reverse *log*))
            (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
            (when (eq (second i) 'start-earn-series)
              (dolist (x (reverse test-list))
                (setf value
                        (if (< (- x start-earn) (- (first i) x))
                          (- x start-earn)
                          (- x (first i)))
                        )
                  (push value accum)
                  )
                  (setf test-list nil)
                  (setf start-earn (first i)))))
        (let* ((avg (if accum (/ (apply #'+ accum) (length accum)) 0))
               (dev (if accum (/ (apply #'+ (mapcar #'(lambda (y) (abs (- y avg))) accum)) (length accum)) 0)))
          (format t " ~8,3F " avg )))
) 

))

(defun scores-indiv (l &optional (blocks 20))
    (dolist (x l)
      (dotimes (i blocks)
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ i 1) (second x)))
        (format t "~8D~%" (second (second (first *log*)))))))

(defun dual-indiv (l &optional (blocks 20))
    (dolist (x l)
      (dotimes (j blocks)
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ j 1) (second x)))
        (let ((total-earn-hit 0)
              (total-spend-hit 0)
              (total-earn-foil 0)
              (total-spend-foil 0)
              (earn-hit-correct 0)
              (earn-foiled 0)
              (spend-hit-correct 0)
              (spend-foiled 0)
              (total-spend-hit-during-earn 0)
              (total-spend-foil-during-earn 0)
              (spend-hit-correct-during-earn 0)
              (spend-foiled-during-earn 0)
              (earn-period nil)
              )
          (dolist (i (Reverse *log*))
            (cond
             ((eq (second i) 'key-earn-error) (incf earn-foiled)(incf total-earn-foil))
             ((eq (second i) 'key-earn-score) (incf earn-hit-correct)(incf total-earn-hit))
             ((eq (second i) 'start-earn-series) (setf earn-period 'inited))
             ((eq (second i) 'press-test) (when (eq earn-period 'inited)(setf earn-period 'yes)))
             ((eq (second i) 'end-earn-series) (setf earn-period nil))
             ((eq (second i) 'earn-hit-done)(incf total-earn-hit))
             ((eq (second i) 'earn-foil-done)(incf total-earn-foil))
             ((eq (second i) 'remove-earn-foil-button)(incf total-earn-foil))
             ((eq (second i) 'remove-earn-hit-button)(incf total-earn-hit))
             ((eq (second i) 'click-spend-error) (incf spend-foiled)(incf total-spend-foil)
              (when (eq earn-period 'yes)(incf total-spend-foil-during-earn)(incf spend-foiled-during-earn)))
             ((eq (second i) 'click-spend-score)(incf spend-hit-correct)(incf total-spend-hit)
              (when (eq earn-period 'yes) (incf total-spend-hit-during-earn)(incf spend-hit-correct-during-earn)))
             ((eq (second i) 'remove-spend-foil-button)(incf total-spend-foil)
              (when (eq earn-period 'yes) (incf total-spend-foil-during-earn))) 
             ((eq (second i) 'remove-spend-hit-button)(incf total-spend-hit)
              (when (eq earn-period 'yes) (incf total-spend-hit-during-earn)))))
          (format t "S~D~S ~5S ~5D ~8,3F~%" (first x) (second x) (second x) (+ j 1)
                  (if (> total-spend-hit-during-earn 0) (/ spend-hit-correct-during-earn total-spend-hit-during-earn) 0))))))

(defun average-time-first-only (l &optional (blocks 20))
  (dolist (x l)
    (dotimes (j blocks)
      (let ((accum nil)(prev-empty t))
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ j 1) (second x)))
        (let ((start-earn -100000) (test-list nil) value)
          (dolist (i (reverse *log*))
            (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
            (when (eq (second i) 'start-earn-series)
              (setf test-list (mapcar #'(lambda (x) 
                                          (if (< (- x start-earn) (- (first i) x))
                                            (- x start-earn)
                                            (- x (first i)))) test-list))
              (let ((min-value 10000)(min-pos-value 10000))
                (dolist (x test-list)
                  (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                  (when (< x min-value)(setf min-value x)))
                (when (and (not (= min-pos-value 10000)) prev-empty) (push min-pos-value accum))
                (if (<= min-value 0) (progn (push min-value accum)(setf prev-empty nil))
                    (setf prev-empty t))
                )
              (setf test-list nil)
              (setf start-earn (first i)))))
        (let* ((avg (if accum (/ (apply #'+ accum) (length accum)) 0))
               (dev (if accum (/ (apply #'+ (mapcar #'(lambda (y) (abs (- y avg))) accum)) (length accum)) 0)))
          (format t "~%S~D ~4S ~5D ~8,3F ~8,3F " (first x) (second x) (1+ j) avg dev))))))



(defun time-first-only (l fname  &optional (start 1)(finish 20))
  (with-open-file (f fname :direction :output :if-exists :error)
  (dolist (x l)
    (let ((count 0))
    (dotimes (j (1+ (- finish start)))
      (let ((accum nil)(prev-empty nil))
        (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ j start) (second x)))
        (let ((start-earn -100000) (end-earn -100000) (test-list nil) value)
          (dolist (i (reverse *log*))
            (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
            (when (eq (second i) 'end-earn-series) 
              (when (not (= end-earn -100000))
                (setf test-list (mapcar #'(lambda (x)
                                        (- x start-earn)) test-list))
               ; (print test-list)
                (setf test-list (remove-if #'(lambda (x) (<= x -6000)) test-list)) ;; remove ambiguous clicks
                (let ((min-value 10000)(min-pos-value 10000))
                  (dolist (x test-list)
                    (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                    (when (< x min-value)(setf min-value x)))
                  (incf count)
                  (format f "~%S~D~S ~4S ~5D ~8,3F ~8,3F" (first x)(second x)(second x) count 
                          (if (< min-value 10000) min-value -9999)
                          (if (< min-pos-value 10000) (- (- (first i) start-earn) min-pos-value) 0))
                  ))
              (setf test-list nil)
              (Setf end-earn (first i)))
            (when (eq (second i) 'start-earn-series) (setf start-earn (first i)))
            )
          (when (> start-earn end-earn) ; test-list
            (setf test-list (mapcar #'(lambda (x)
                                        (- x start-earn)) test-list))
            ; (print test-list)
                (setf test-list (remove-if #'(lambda (x) (<= x -6000)) test-list)) ;; remove ambiguous clicks
                (let ((min-value 10000)(min-pos-value 10000))
                  (dolist (x test-list)
                    (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                    (when (< x min-value)(setf min-value x)))
                  (incf count)
                  (format f "~%S~D~S ~4S ~5D ~8,3F ~8,3F" (first x)(second x)(second x) count 
                          (if (< min-value 10000) min-value -9999)
                          (if (< min-pos-value 10000) (- 6000 min-pos-value) 0))
                  ))
)))))))


(defun histo+ (l)
  (let ((total (make-array '(25 4) :initial-element 0))
        (t-block-avg (make-array '(4) :initial-element 0))
        (t-block-abs (make-array '(4) :initial-element 0))
        )
    (dolist (x l)
      (dotimes (i 4)
        (let ((bins (make-array '(25) :initial-element 0))
              (block-avg 0)
              (block-abs 0)
              (count 0))
          (dotimes (j 5)
            (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ (* i 5) j 1) (second x)))
            (let ((start-earn -100000) (test-list nil) (prev-empty nil))
              (dolist (i (reverse *log*))
            (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
            (when (eq (second i) 'start-earn-series)
              (setf test-list (mapcar #'(lambda (x) 
                                          (if (> (- (first i) x) 7000)
                                            (- x start-earn)
                                            (- x (first i)))) test-list))
              (setf test-list (remove-if #'(lambda (x) (<= x -5000)) test-list)) ;; remove ambigous clicks
              (let ((min-value 10000)(min-pos-value 10000))
                (dolist (x test-list)
                  (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                  (when (< x min-value)(setf min-value x)))
                (when (and (not (= min-pos-value 10000)) prev-empty)
                    (let ((bin (truncate (/ (+ min-pos-value 6250) 500))))     
                      (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                                  ))
                      (setf block-avg (+ block-avg min-pos-value))
                      (setf block-abs (+ block-abs min-pos-value))
                      (incf count)
                  ))
                (when (and (= min-pos-value 10000) prev-empty)
                   (incf count)
                   (incf (aref bins 0)))
                (if (<= min-value 0) (progn 
                                       (let ((bin (truncate (/ (+ min-value 6250) 500))))     
                                         (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                                                     ))
                      (setf block-avg (+ block-avg min-value))
                      (setf block-abs (- block-abs min-value))
                      (incf count)
                  )
                                            (setf prev-empty nil))
                    (setf prev-empty t))
                )
              (when (= start-earn -100000) (setf prev-empty nil))
                  (setf test-list nil)
                  (setf start-earn (first i))))))
          (dotimes (j 25) 
            (setf (aref total j i) (if (= count 0) (aref total j i) (+ (aref total j i) (/ (aref bins j) count)))))
          (setf (aref t-block-avg i) (if (= count 0) (aref t-block-avg i)(+ (aref t-block-avg i) (/ block-avg count))))
          (setf (aref t-block-abs i) (if (= count 0) (aref t-block-abs i)(+ (aref t-block-abs i) (/ block-abs count))))

          )))
    (dotimes (i 4)
      (format t "~%Block ~D~%" (1+ i))
      (dotimes (j 25)
        (format t " ~8,4F~%" (/ (aref total j i) (length l)))))
    (format t "~%Avg Times~%")
    (dotimes (i 4) (format t "~8,4F~%" (/ (aref t-block-avg i) (length l))))
    (format t "~%Abs Times~%")
    (dotimes (i 4) (format t "~8,4F~%" (/ (aref t-block-abs i) (length l))))
    
))

(defun histo++ (l &optional (raw nil))
  (let ((total (make-array '(13 4) :initial-element 0))
 ;       (t-block-avg (make-array '(4) :initial-element 0))
 ;       (t-block-abs (make-array '(4) :initial-element 0))
        )
    (dolist (x l)
      (let ((count 0))
      (dotimes (i 4)
        (let ((bins (make-array '(13) :initial-element 0)))

          (dotimes (j 5)
            (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ (* i 5) j 1) (second x)))
            (let ((start-earn -100000) (test-list nil) (prev-empty nil))
              (dolist (i (reverse *log*))
            (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
            (when (eq (second i) 'start-earn-series)
              (setf test-list (mapcar #'(lambda (x) 
                                          (if (> (- (first i) x) 7000)    ; if the click is before the end of the last period
                                            (if (> (- x start-earn) 0) (- x start-earn) 0)
                                            ; take as time the time after the start of the previous period (= positive number)
                                            (- x (first i)))) test-list)) ; otherwise the time until the next period (now, a negative number)
              (setf test-list (remove-if #'(lambda (x) (<= x -5000)) test-list)) ;; remove ambigous clicks
              (let ((min-value 10000)(min-pos-value 10000))
                (dolist (x test-list)
                  (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                  (when (< x min-value)(setf min-value x)))
;                (when (and (not (= min-pos-value 10000)) prev-empty)
                (if (= min-pos-value 10000) (setf min-pos-value 0)(setf min-pos-value (- 6000 min-pos-value))) ; convert to time until end
                (when  prev-empty
                (let ((bin (truncate (/ (+ min-pos-value 0) 500))))     
                  (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                              ))
;                  (setf block-avg (+ block-avg min-pos-value))
;                  (setf block-abs (+ block-abs min-pos-value))
                  (incf count)
                  (when raw (format t "~%S~D ~4S ~5D ~8,3F  " (first x) (second x) count min-pos-value))   
                  ))
)
                 
              (if (= start-earn -100000) (setf prev-empty nil)(setf prev-empty t))
              
                  (setf test-list nil)
                  (setf start-earn (first i)))) 
              (setf test-list (mapcar #'(lambda (x) 
                                            (- x start-earn)  ; take as time the time after the start of the previous period (= positive number)
                                            ) test-list)) ; otherwise the time until the next period (now, a negative number)
              (setf test-list (remove-if #'(lambda (x) (<= x -5000)) test-list)) ;; remove ambigous clicks
              (let ((min-value 10000)(min-pos-value 10000))
                (dolist (x test-list)
                  (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                  (when (< x min-value)(setf min-value x)))
;                (when (and (not (= min-pos-value 10000)) prev-empty)
                (if (= min-pos-value 10000) (setf min-pos-value 0)(setf min-pos-value (- 6000 min-pos-value))) ; convert to time until end
                (when  prev-empty
                (let ((bin (truncate (/ (+ min-pos-value 0) 500))))     
                  (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                              ))
;                  (setf block-avg (+ block-avg min-pos-value))
;                  (setf block-abs (+ block-abs min-pos-value))
                  (incf count)
                  (when raw (format t "~%S~D ~4S ~5D ~8,3F  " (first x) (second x) count min-pos-value))   
                  ))
)))
          (dotimes (j 13) 
            (setf (aref total j i) (if (= count 0) (aref total j i) (+ (aref total j i) (/ (aref bins j) count)))))
 
          ))))
    (dotimes (i 4)
      (format t "~%Block ~D~%" (1+ i))
      (dotimes (j 13)
        (format t " ~8,4F~%" (/ (aref total j i) (length l)))))
    
))

(defun histo++exp1 (l &optional (raw nil))
  (let ((total (make-array '(13 4) :initial-element 0))
 ;       (t-block-avg (make-array '(4) :initial-element 0))
 ;       (t-block-abs (make-array '(4) :initial-element 0))
        )
    (dolist (x l)
      (dotimes (i 3)
      (let ((count 0)
            (base (* i 45)))
        (let ((bins (make-array '(13) :initial-element 0)))

          (dotimes (j 5)
            (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ (* i 5) j 1) (second x)))
            (let ((start-earn -100000) (test-list nil) (prev-empty nil))
              (dolist (i (reverse *log*))
            (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
            (when (eq (second i) 'start-earn-series)
              (setf test-list (mapcar #'(lambda (x) 
                                          (if (> (- (first i) x) 7000)    ; if the click is before the end of the last period
                                            (- x start-earn)  ; take as time the time after the start of the previous period (= positive number)
                                            (- x (first i)))) test-list)) ; otherwise the time until the next period (now, a negative number)
              (setf test-list (remove-if #'(lambda (x) (<= x -5000)) test-list)) ;; remove ambigous clicks
              (let ((min-value 10000)(min-pos-value 10000))
                (dolist (x test-list)
                  (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                  (when (< x min-value)(setf min-value x)))
;                (when (and (not (= min-pos-value 10000)) prev-empty)
                (if (= min-pos-value 10000) (setf min-pos-value 0)(setf min-pos-value (- 6000 min-pos-value))) ; convert to time until end
                (when (not (= start-earn -100000))
                (let ((bin (truncate (/ (+ min-pos-value 0) 500))))     
                  (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                              ))
;                  (setf block-avg (+ block-avg min-pos-value))
;                  (setf block-abs (+ block-abs min-pos-value))
                  (incf count)
                  (when raw (format t "~%S~D ~4S ~5D ~8,3F  " (first x) (second x) (+ base count) min-pos-value))                                           
                  )))

                  (setf test-list nil)
                  (setf start-earn (first i))))))
          (dotimes (j 13) 
            (setf (aref total j i) (if (= count 0) (aref total j i) (+ (aref total j i) (/ (aref bins j) count)))))
 
          ))))
    (dotimes (i 3)
      (format t "~%Block ~D~%" (1+ i))
      (dotimes (j 13)
        (format t " ~8,4F~%" (/ (aref total j i) (length l)))))
    
))


(defun avg (l)
  (let ( ; (total (make-array '(25 3) :initial-element 0))
        (t-block-avg (make-array '(20) :initial-element 0))
        (t-block-abs (make-array '(20) :initial-element 0))
        )
    (dolist (x l)
      (dotimes (i 20)
        (let ((bins (make-array '(25) :initial-element 0))
              (block-avg 0)
              (block-abs 0)
              (count 0))
            (load (format nil "MoneyExp1-S~D-T~D-~S" (first x) (+ i 1) (second x)))
            (let ((start-earn -100000) (test-list nil) (prev-empty nil))
              (dolist (i (reverse *log*))
            (when (and (eq (second i) 'press-test)(not (= start-earn -100000))) (push (first i) test-list))
            (when (eq (second i) 'start-earn-series)
              (setf test-list (mapcar #'(lambda (x) 
                                          (if (> (- (first i) x) 7000)
                                            (- x start-earn)
                                            (- x (first i)))) test-list))
              (setf test-list (remove-if #'(lambda (x) (<= x -5000)) test-list)) ;; remove ambigous clicks
              (let ((min-value 10000)(min-pos-value 10000))
                (dolist (x test-list)
                  (when (and (> x 0)(< x min-pos-value)) (setf min-pos-value x))
                  (when (< x min-value)(setf min-value x)))
                (when (and (not (= min-pos-value 10000)) prev-empty)
                    (let ((bin (truncate (/ (+ min-pos-value 6250) 500))))     
                      (incf (aref bins (cond ((< bin 0) 0)((> bin 24) 24) (t bin))
                                  ))
                      (setf block-avg (+ block-avg min-pos-value))
                      (setf block-abs (+ block-abs min-pos-value))
                      (incf count)
                  ))
                (if (<= min-value 0) (progn 
                                  
                      (setf block-avg (+ block-avg min-value))
                      (setf block-abs (- block-abs min-value))
                      (incf count)
                  
                      (setf prev-empty nil))
                    (setf prev-empty t))
                )
              (when (= start-earn -100000) (setf prev-empty nil))
                  (setf test-list nil)
                  (setf start-earn (first i)))))
          (setf (aref t-block-avg i) (if (= count 0) (aref t-block-avg i)(+ (aref t-block-avg i) (/ block-avg count))))
          (setf (aref t-block-abs i) (if (= count 0) (aref t-block-abs i)(+ (aref t-block-abs i) (/ block-abs count))))

          )))
    (format t "~%Avg Times~%")
    (dotimes (i 20) (format t "~8,4F~%" (/ (aref t-block-avg i) (length l))))
    (format t "~%Abs Times~%")
    (dotimes (i 20) (format t "~8,4F~%" (/ (aref t-block-abs i) (length l))))
    
))

#|
(setf *aa* '((1 aa)(9 aa)(13 aa)(17 aa)(21 aa)(25 aa)(29 aa))) ;; removed 5
(setf *ll* '((6 ll)(10 ll)(14 ll)(18 ll)(22 ll)(26 ll)(30 ll))) ;; removed 2
(setf *al* '((3 al)(7 al)(11 al)(15 al)(19 al)(23 al)(27 al)(31 al)))
(setf *la* '((4 la)(12 la)(16 la)(20 la)(24 la)(28 la)(32 la)))  ;; removed 8

(setf *all* '((1 aa)(3 al)(4 la)(6 ll)(7 al)(9 aa)(10 ll)(11 al)(12 la)(13 aa)(14 ll)(15 al)(16 la)
              (17 aa)(18 ll)(19 al)(20 la)(21 aa)(22 ll)(23 al)(24 la)(25 aa)(26 ll)(27 al)(28 la)
              (29 aa)(30 ll)(31 al)(32 la)))


(setf *ll* '((101 LL) (102 LL) (103 LL) (104 LL) (105 LL) (106 LL) (107 LL) (108 LL) (109 LL) (110 LL)))
(setf *aa* '((101 aa)(102 aa)(103 aa)(104 aa)(105 aa)(106 aa)(107 aa)(108 aa)(109 aa)(110 aa)))
(setf *la* '((101 la)(102 la)(103 la)(104 la)(105 la)(106 la)(107 la)(108 la)(109 la)(110 la)))
(Setf *al* '((101 al)(102 al)(103 al)(104 al)(105 al)(106 al)(107 al)(108 al)(109 al)(110 al)))
;(setf *model* '((101 ll)(102 ll)(103 ll)(104 ll)(105 ll)(106 ll)(107 ll)))
|#
#|
; experiment 1 settings
(setf *aa* '((1 aa)(5 aa)(9 aa)(13 aa)(17 aa)(21 aa)(25 aa)(29 aa))) ;; removed 5
(setf *ll* '((2 ll)(6 ll)(10 ll)(14 ll)(18 ll)(22 ll)(26 ll)(30 ll))) ;; removed 2
(setf *al* '((3 al)(7 al)(11 al)(15 al)(19 al)(23 al)(27 al)(31 al)))
(setf *la* '((4 la)(8 la)(12 la)(16 la)(20 la)(24 la)(28 la)(32 la)))  ;; removed 8

(setf *all* (append *aa* *ll* *al* *la*))
|#




(setf *ll* '((4 ll)(8 ll)(12 ll)(16 ll)(20 ll)(24 ll)(28 ll)(32 ll)(36 ll)(40 ll)))
(setf *aa* '((1 aa)(5 aa)(9 aa)(13 aa)(17 aa)(21 aa)(25 aa)(29 aa)(33 aa)(37 aa)))

(setf *la* '((3 la)(7 la)(11 la)(15 la)(19 la)(23 la)(27 la)(31 la)(35 la)(39 la))) ; subject 3 doesn't try in block 3-4
(setf *al* '((2 al)(6 al)(10 al)(14 al)(18 al)(22 al)(26 al)(30 al)(34 al)(38 al)))




(defun set-vars (n)
  (setf *ll* nil *aa* nil *la* nil *al* nil)
  (dotimes (i n)
    (push (list (+ 100 (- n i)) 'll) *ll*)
    (push (list (+ 100 (- n i)) 'la) *la*)
    (push (list (+ 100 (- n i)) 'al) *al*)
    (push (list (+ 100 (- n i)) 'aa) *aa*)))

(defun do-ll-n (n &optional (m 1))
  (dotimes (i n)(do-ll-model (+ m i)))
  (set-vars (+ m n -1))
  (histo *ll*)
  (scores *ll*)
  (dual *ll*))

(defun do-aa-n (n &optional (m 1))
  (dotimes (i n)(do-aa-model (+ m i)))
  (set-vars (+ m n -1))
  (histo *aa*)
  (scores *aa*)
  (dual *aa*))

(defun do-al-n (n &optional (m 1))
  (dotimes (i n)(do-al-model (+ m i)))
  (set-vars (+ m n -1))
  (histo *al*)
  (scores *al*)
  (dual *al*))

(defun do-la-n (n &optional (m 1))
  (dotimes (i n)(do-la-model (+ m i)))
  (set-vars (+ m n -1))
  (histo *la*)
  (scores *la*)
  (dual *la*))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Temporal buffer code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar *time-noise*)
(defvar *time-mult*)
(defvar *start-increment*)
(defvar *master-start-increment*)
(defvar *temporal*)

(setf *time-noise* 0.015 *time-mult* 1.1 *master-start-increment* .011) ; *time-noise* is .015 by default

;;; The following production can be used to simulate the behavior
;;; of the temporal buffer using lisp functions
;;; They are not necessary for the buffer itself

;(defun start-subject ()
;  (setf *start-increment* (+ *master-start-increment* (noise (* *master-start-increment* *time-noise* 5)))))

(defun start-subject ()
  (setf *start-increment* (+ *master-start-increment* -.001 (* .002 (random 1.0)))))



(defun temporal ()
;  (format t "~%Call to temporal~%")
  (eval `(dm ,*temporal*)))

(defun get-temporal-content (arguments)
;  (format t "~%Get-temporal with ~S~%" arguments)
  "Argument is a list containing the name of the buffer and its global variable."
  (let* ((name (pop arguments))
         (buffer (pop arguments))
         (chunk (symbol-value buffer)))
    (signal-output *exact-matching-trace* "BUFFER ~A holds CHUNK ~A." name chunk)    
    (symbol-value buffer)))

(defun clear-temporal (arguments)
;  (format t "~%clear-temporal with ~S~%" arguments)
  "Argument is a list containing the name of the buffer and its global variable."
  (let ((name (pop arguments))
        (buffer (pop arguments)))
    (signal-output *dm-trace* "Clearing BUFFER ~A." name)    
    (setf (symbol-value buffer) nil)))

(defun create-temporal-chunk (arguments)
  "Arguments are a list containing the name of the buffer, the global variable
   holding the buffer contents, then the chunk description with isa and slots.
   THIS LAST ARGUMENT MUST BE FUNCALLED TO RESOLVE THE VALUE OF PRODUCTION VARIABLES."
;  (format t "~%Create-temporal with ~S~%" arguments)
  (let* ((name (pop arguments))
         (buffer (pop arguments))
         (chunk (new-name-fct name))
         (specs (funcall (pop arguments))))
    (signal-output *dm-trace* "Creating CHUNK ~A in BUFFER ~A with specs ~S."
                   chunk name specs)
    (add-dm-fct (list (cons chunk '(isa time ticks 0))))
    (let ((tick (+ *start-increment* (noise (* *time-noise* *start-increment*)))))
      (queue-command :time tick
                     :where :cognition :command 'next-time-tick :randomize nil :params (list tick chunk)))
    (setf (symbol-value buffer) (get-wme chunk))))

(defun modify-temporal-chunk (arguments)
  "Arguments are a list containing the name of the buffer, the global variable
   holding the buffer contents, then the chunk description with slots but no isa.
   THIS LAST ARGUMENT MUST BE FUNCALLED TO RESOLVE THE VALUE OF PRODUCTION VARIABLES."
;  (format t "~%Modify-temporal with ~S~%" arguments)
   (let* ((name (pop arguments))
         (buffer (pop arguments))
         (chunk (symbol-value buffer))
         (specs (funcall (pop arguments))))
    (signal-output *dm-trace* "Modifying CHUNK ~A in BUFFER ~A with specs ~S."
                   chunk name specs)
    (mod-chunk-fct chunk specs)))

(defgeneric next-time-tick (cog-mod tick chunk))

(defmethod next-time-tick ((cog-mod cognition-module) tick chunk)
  (when (and (wmep *temporal*) (eq chunk (wme-name *temporal*)))
;    (format t "~%Entering next-time-tick with ~6,3F and ~D" tick (get-slot-value *temporal* 'ticks))
    (setf tick (* tick *time-mult*))
    (setf tick (+ tick (noise (* *time-noise* tick))))
    (mod-chunk-fct chunk (list 'ticks (1+ (get-slot-value *temporal* 'ticks))))
    (queue-command :time tick :where :cognition :command 'next-time-tick :randomize nil :params (list tick chunk))))

;;; Visual module hack to detect errors in visual
;;; The catch is that **visual-error** has to be cleared!

(setf **visual-error** nil)

(defmethod encoding-complete ((vis-mod vision-module) (loc-dmo dmo) scale)
  (declare (symbol scale))
  (setf **visual-error** nil)
  (set-attributes loc-dmo '(attended t))
  (setf (moving-attention vis-mod) nil)
  (change-state vis-mod :exec 'free :proc 'free)
  ;(setf (current-marker vis-mod) loc-dmo)
  (let ((return-obj (get-obj-at-location vis-mod loc-dmo scale)))
    (unless return-obj
      ;(clear-loc loc-dmo)
      (clear-attended vis-mod)
      (setf **visual-error** t)
      (return-from encoding-complete nil))
    (set-attended vis-mod return-obj)
    (when (print-view-p vis-mod)
      (pm-output nil "Vision sees ~S" (id return-obj)))
    ;(format t "~% <> MOVE-ATTENTION generated DMO ~S~%" (id return-obj)))
    return-obj))



(defun -retrieval> ()
	(and (null *retrieval*) (null *retrieval-scheduler*)))

(defun -visual-location> ()
	(null *visual-location*))
	
(defun -visual> ()
	(and (not (currently-attended (vision-mod)))))


(defmethod stuff-visloc-buffer ((vis-mod vision-module))
  (unless (and *visual-location* (not (eq (wme-name *visual-location*) 'failure)))
    (awhen (find-current-locs-with-spec vis-mod (default-spec vis-mod))
      (setf *visual-location*
            (construct-location vis-mod 
                                (random-item (objs-max-slotval it 'tstamp))
                                (default-spec vis-mod))))))


;;; We are going to hack credit assignment in order to assign credit based on
;;; score.
;;; On a success we do not add one, but instead call a function called payoff 
;;; which produces a value between zero and 1
;;;

(defvar *last-score*)

(defun payoff (&optional (done nil))
  (let ((result
         (max 0.0 (- 40 (/ (- *money* *last-score*) 10)))
         ))
    (when done (setf *last-score* *money*))
    result))


(defun learn-parameters (success failure &key (latency 0.0) (instantiation *instantiation*))
  "Applies the parameters learning equations to instantiation.  The success and/or
   failure of the goal has already been determined and is passed along with the latency
   of this instantiation and the instantiation itself (if it exists)."
  (let ((production (when instantiation (instantiation-production instantiation))))
    (cond ((and success failure)  ;; clear the history
           (setf *previous-instantiations* nil))
          ((or success failure)  ;; register success or failure for all productions
           (when production
             (push (cons production (- *time* latency)) *previous-instantiations*))
           (dolist (production-time *previous-instantiations*)
             (let* ((previous-production (car production-time))
                    (previous-time (cdr production-time))
                    (effort (- *time* previous-time)))
               (add-reference (if success (production-successes previous-production)
                                  (production-failures previous-production))
                              (not (numberp *parameters-learning*))
                              1.0 previous-time)            
               (add-reference (production-efforts previous-production)
                              (not (numberp *parameters-learning*))
                              (payoff) (payoff))  ;;; instead of effort we use payoff
               (recompute-production-parameters previous-production)))
           (payoff t) ;;; we tell payoff we're done
           (setf *previous-instantiations* nil))
          (t  ;; simply add the production
           (when production
             (push (cons production (- *time* latency)) *previous-instantiations*))))))

;;; Hack production effort to make the time to retrieve an addition a random
;;; value between 200 ms and 350 ms

(defun find-addition-prod (production)
  (when (= (production-effort production) 0.3)
    (+ 0.25 (* .2 (random 1.0))))) ; was (+ 0.25 (* .25 (random 1.0)))

(defmacro production-action-cost (production)
  "The action cost of a production is either the effort if specified or C."
  `(or (and *addition* (find-addition-prod ,production))
       (get-functional-parameter (production-effort ,production))
       (production-c ,production)))

(defun matching-cycle (&optional (cycles nil) (retrieval-threshold *retrieval-threshold*) 
                                 (utility-threshold *utility-threshold*))
  "Implements the production matching (lhs) cycle called by run-fct and the PM scheduler.
   Cycles is the time limit for the matching cycle if applicable.
   Returns the latency of the action phase if any, nil otherwise."
  (let ((matching-latency 0.0)
        (failed-latencies 0.0)
        (action nil))
    (when (and *retrieval-threshold* *activation-noise* *threshold-noise*)
      (let ((noise (noise *activation-noise*)))
        (setf *retrieval-threshold* (+ retrieval-threshold noise))
        (signal-output *activation-trace* "   Adding noise ~6,3F to retrieval threshold for a total of ~6,3F"
                       noise *retrieval-threshold*)))
    (when (and *utility-threshold* *exp-gain-noise* *threshold-noise*)
      (let ((noise (noise *exp-gain-noise*)))
        (setf *utility-threshold* (+ utility-threshold noise))
        (signal-output (or *exact-matching-trace* *partial-matching-trace*)
                       "   Adding noise ~6,3F to utility threshold for a total of ~6,3F"
                       noise *utility-threshold*)))
    (generate-all-instantiations)
    (when *matches-trace*
      (pprint-instantiations *conflict-set* *matches-trace* *matches-trace*))
    (setf *instantiation* nil)
    (when *conflict-set-hook-fn*
      (multiple-value-bind (instantiations latency)
                           (funcall *conflict-set-hook-fn* *conflict-set*)
        (declare (ignore latency))
        (when instantiations ; if returns nil, then continue unchanged
          (cond ((listp instantiations) ; if returns a list, then interpret as new cset
                 (setf *conflict-set* instantiations))
                (t ; otherwise, restrict the conflict set to selected instantiation
                 (setf *conflict-set* (list instantiations)))))))
    (unless *instantiation*        
      (multiple-value-setq (*instantiation* matching-latency failed-latencies)
        (choose-instantiation)))
    ;;; the following used to be part of the action cycle but really belongs in matching
    (cond (*instantiation*
           (let ((production (instantiation-production *instantiation*)))
             (cond ((and *abort-instantiation* *enable-rational-analysis*
                         (floatp cycles)
                         (> (+ *time* matching-latency)
                            cycles))
                    (signal-output *latency-trace* "Time ~6,3F: ~A Aborted" *time* production))
                   (t
                    (when (or *base-level-learning* *associative-learning*
                              *strength-learning*)
                      (learn-matching))
                    (when *enable-rational-analysis*
                      (incf *time* matching-latency)
                      (when (plusp matching-latency)
                        (signal-output *latency-trace* "Latency ~6,3F: ~A Matching"
                                       matching-latency production)))
                    (signal-output *cycle-trace* "Time ~6,3F: ~A Selected" *time* production)
                    (setf (instantiation-latency *instantiation*)
                          (- matching-latency failed-latencies))
                    (setf action (production-action-cost production))))))
          ((and *enable-rational-analysis* *pop-upon-failure* (null *retrieval-scheduler*))
           (incf *time* matching-latency)
           (when (plusp matching-latency)
             (signal-output *latency-trace* "Latency ~6,3F: Failure Matching"
                            matching-latency))
           (setf action *default-action-time*))
          (t
           (signal-output *latency-trace* "Time ~6,3F: No Instantiation Found." *time*)))
    action))

;;; Start of the model

(clear-all)
(pm-reset)
(sgp :esc t :rt -1.0 :lf 0.25 :ans 0.1 :bll 0.5 :pl t :egs 0.5 :ut 2.0 :v t :g 40 :epl t :ol 3)
(setf *pc-speed* 0.2)
(define-buffer temporal *temporal*
  :equal-lhs get-temporal-content
  :plus-rhs create-temporal-chunk
  :equal-rhs modify-temporal-chunk
  :minus-rhs clear-temporal)

(defparameter *buffer-lis* '(goal retrieval visual-location visual visual-state manual manual-state aural aural-state
                                  aural-location vocal vocal-state temporal
                                   intentional)
    "List of buffer names")

(defparameter *=buffer-lis* (mapcar '=buffer *buffer-lis*)
    "List of =buffer> names")

(defparameter *-buffer-lis* (mapcar '-buffer *buffer-lis*)
    "List of -buffer> names")

(defparameter *+buffer-lis* (mapcar '+buffer *buffer-lis*)
    "List of +buffer> names")

(defparameter *variable-lis* (mapcar 'to-variable *buffer-lis*)
    "List of =buffer names")

(defparameter *header* (append *=buffer-lis* *-buffer-lis* *+buffer-lis*))
 (chunk-type time ticks)

(set-letters)

(chunk-type goal state time timestate gamble stimulus)
(chunk-type response arg1 answer)

(add-dm 
        (resp1 isa response arg1 "a" answer yes)
        (resp2 isa response arg1 "b" answer no)
        (resp3 isa response arg1 "e" answer yes)
        (resp4 isa response arg1 "d" answer no))

(sdp :creation-time -1000 :references 50)

(add-dm (goal1 isa goal state start))
(goal-focus goal1)

(p start-task
   =goal>
     isa goal
     state start
==>
   =goal>
     state click-test)

(spp start-task :success t :failure t) ;;; clear out old stuff

;;; Time estimation stuff


(p click-test-look
   =goal>
     isa goal
     state click-test
   =visual-state>
     isa module-state
     modality free
==>
   !eval! (setf **visual-error** nil)
   +visual-location>
     isa visual-location
     screen-x (within 350 450)
     screen-y (within 35 75)
     kind text
   =goal>
     state click-test-move)

(p click-test-look-on-time
   =goal>
     isa goal
     state main
     timestate click-on-time
     time =time
   =visual-state>
     isa module-state
     modality free
   =temporal>
     isa time
     >= ticks =time
==>
   !eval! (setf **visual-error** nil)
   +visual-location>
     isa visual-location
     screen-x (within 350 450)
     screen-y (within 35 75)
     kind text
   =goal>
     state click-test-move
     timestate nil
)

(p click-test-move
   =goal>
     isa goal
     state click-test-move
   =visual-location>
     isa visual-location
   =visual-state>
     isa module-state
     modality free
   =manual-state>
     isa module-state
     preparation free
==>
   +visual>
     isa visual-object
     screen-pos =visual-location
   +manual>
     isa move-cursor
     loc =visual-location
   =goal>
     state click-test-click)

(p click-test-click
   =goal>
     isa goal
     state click-test-click
     timestate nil
   =visual-state>
     isa module-state
     modality free
   =visual>
     isa text
   =manual-state>
     isa module-state
     preparation free
==>
   +manual>
     isa click-mouse
   +temporal>
     isa time  ;;; initiate temporal because we expect something to happen due to the click
   =goal>
     state main
     timestate waiting
   +visual-location> isa visual-location attended new kind text
   )

(p click-test-click-on-repair
   =goal>
     isa goal
     state click-test-click
     timestate repair
   =visual-state>
     isa module-state
     modality free
   =visual>
     isa text
   =manual-state>
     isa module-state
     preparation free
==>
    -temporal>
   +manual>
     isa click-mouse
   =goal>
     state main
   +visual-location> isa visual-location attended new kind text
   )

;;; Productions for state "main"
;;; Relies on buffer stuffing

(p visual-location-left
   =goal>
     isa goal
     - state left-box
;     - timestate waiting
   =visual-location>
     isa visual-location
     kind text
     < screen-x 300
   =visual-state>
     isa module-state
     modality free
==>
   +visual>
     isa visual-object
     screen-pos =visual-location
   -retrieval>
   =goal>
     state left-box)


(p visual-location-right
   =goal>
     isa goal
     state main
   =visual-location>
     isa visual-location
     kind text
     > screen-x 420
   =visual-state>
     isa module-state
     modality free
==>
   +visual>
     isa visual-object
     screen-pos =visual-location
   -retrieval>
   =goal>
     state right-box)

(p visual-location-in-the-middle
   =goal>
     isa goal
     state main
   =visual-location>
     isa visual-location
     kind text
     > screen-x 299
     < screen-x 421
   =visual-state>
     isa module-state
     modality free
==>
   =goal>
      state middle
   +visual>
     isa visual-object
     screen-pos =visual-location
 )
    
(p process-stimulus-just-found-test
   =goal>
     isa goal
     state middle
   =visual-state>
     isa module-state
     modality free
   =visual>
     isa text
     value "test"
==>
   =goal>
     state main
   +visual-location> isa visual-location attended new kind text
   )

;;; Left-box processing. First the special case of when the word "end" turns up

(p process-stimulus-end
   =goal>
     isa goal
     state left-box
   =visual-state>
     isa module-state
     modality free
   =visual>
     isa text
     value "end"
==>
   +temporal>
     isa time
   +visual-location> isa visual-location attended new kind text 
   =goal>
     state main
     timestate wait-click
     time 30
)

(spp process-stimulus-end :success t)

(p find-a-time
   =goal>
     isa goal
     state main
     timestate wait-click
     time =old-time
   !eval! (-retrieval>)
   =temporal>
     isa time
     > ticks =old-time
     ticks =time
   =manual-state>
     isa module-state
     modality free
==>
   =goal>
     time =time
     gamble nil
   +retrieval>
     isa goal
     state done
      > time =time
      <= time (!eval! (+ =time 1 (random 4)))
)

(p try-time-on-success
   =goal>
     isa goal
     state main
     timestate wait-click
   =retrieval>
     isa goal
     timestate success
     time =time 
==>
   -retrieval>
   =goal>
;     state main
     timestate click-on-time
     time =time)

(p dont-try-time-on-failure
   =goal>
     isa goal
     state main
     timestate wait-click
     gamble nil
   =retrieval>
     isa goal
     timestate failure
     time =time 
==>
   
   =goal>
     time =time
     gamble no
)

(p on-retrieval-failure-gamble
   =goal>
     isa goal
     state main
     timestate wait-click
     gamble nil
     time =time
   =retrieval>
     isa error
  !eval! (< (random 1.0) (/ (* =time =time) 6000))

==>
   =goal>
     state click-test
     timestate nil)

(p on-retrieval-failure
   =goal>
     isa goal
     state main
     timestate wait-click
     time =time
     gamble nil
   =retrieval>
     isa error
==>
   =goal>
      state main
      gamble no
)

(spp on-retrieval-failure :failures 100000 :successes 10000)

;;; Left-box processing when "high" turns up

(p process-stimulus-left-high
   =goal>
     isa goal
     state left-box
     stimulus =stim
   =visual-state>
     isa module-state
     modality free
   =visual>
     isa text
     value "high"
==>
   =goal>
     state done
     timestate success
     stimulus nil
   +goal>
     isa goal
     state main
     stimulus =stim
   +visual-location> isa visual-location attended new kind text)

(p process-stimulus-left-high-2
   =goal>
     isa goal
     state left-box
     stimulus nil
   =visual-state>
     isa module-state
     modality free
   =visual>
     isa text
     value "high"
==>
   =goal>
     state done
     timestate success
     stimulus nil
   +goal>
     isa goal
     state main
   +visual-location> isa visual-location attended new kind text)
     

;;; Left-box processing of normal stimuli

(p process-stimulus-left
   =goal>
     isa goal
     state left-box
   =visual-state>
     isa module-state
     modality free
   =visual-location>
     isa visual-location
   =visual>
     isa text
     - value "end"
     - value "high"
     value =stim
     screen-pos =visual-location
==>
   +retrieval>
     isa response
     arg1 =stim
   +visual-location> isa visual-location attended new kind text
   
)

(p decide-response-left
   =goal>
     isa goal
     state left-box
     - timestate waiting
   =retrieval>
     isa response
     answer yes
   =manual-state>
     isa module-state
     preparation free
==>
   +manual>
     isa punch
     hand left
     finger index
   -retrieval>
   =goal>
     state main)

(p decide-no-response-left
   =goal>
     isa goal
     state left-box
     - timestate waiting
   =retrieval>
     isa response
     answer no
==>
   -retrieval>
   =goal>
     state main
)

(p decide-no-response-left-2
   =goal>
     isa goal
     state left-box
   =retrieval>
     isa error
==>
   -retrieval>
   =goal>
     state main
)

;;; Right box processing

(p process-stimulus-right
   =goal>
     isa goal
     state right-box
   =visual-state>
     isa module-state
     modality free
   =visual-location>
     isa visual-location
   =visual>
     isa text
     value =stim
     screen-pos =visual-location
   =manual-state>
     isa module-state
     preparation free
   !eval! (-retrieval>)
==>
   =goal>
     stimulus =stim
   +retrieval>
     isa response
     arg1 =stim
   +manual>
     isa move-cursor
     object =visual
   +visual-location> isa visual-location attended new kind text
   
)

(p decide-response-right
   =goal>
     isa goal
     state right-box
   =retrieval>
     isa response
     answer yes
   =manual-state>
     isa module-state
     preparation free
==>
   +manual>
     isa click-mouse
   -retrieval>
   =goal>
     state main
     stimulus nil)

(p decide-no-response-right
   =goal>
     isa goal
     state right-box
   =retrieval>
     isa response
     answer no
==>
   -retrieval>
   =goal>
     state main
     stimulus nil
)

(p decide-no-response-right-2
   =goal>
     isa goal
     state right-box
   =retrieval>
     isa error
==>
   -retrieval>
   =goal>
     state main
     stimulus nil
)

(p pick-up-right-stimulus
   =goal>
     isa goal
     state main
     stimulus =stim
==>
   +retrieval>
     isa response
     arg1 =stim
   =goal>
     state right-box
     stimulus nil)

(p clear-visual-errors-rb
   =goal>
     isa goal
     state right-box
   =visual-state>
     isa module-state
     modality free
   !eval! **visual-error**
==>
   +visual-location> isa visual-location attended new kind text
   =goal>
     state main
   !eval! (setf **visual-error** nil)
)


(p clear-visual-errors-lb
   =goal>
     isa goal
     state left-box
   =visual-state>
     isa module-state
     modality free
   !eval! **visual-error**
==>
   +visual-location> isa visual-location attended new kind text
   =goal>
     state main
   !eval! (setf **visual-error** nil)
)


(p clicked-but-no-response
   =goal>
     isa goal
     state main
     timestate waiting
     time =time
   =temporal>
     isa time
     > ticks 30  ;;; roughly 2 seconds (?)
==>
   =goal>
     state done
     timestate failure
   +goal>
     isa goal
     state click-test
     timestate repair
     time 30
   -temporal>)

